-- by Frvetz
-- Contact: ExtendedVehicleMaintenance@gmail.com
-- Date 23.10.2020

--[[
Changelog Version 1.0.0.1:
- Description adapted
- New icon
- Day limit adapts to the Seasons-Mod
- Engine may fail to start when starting the motor after reaching the limit
- Engine can stall if engine load is too high
- Bugs fixed when servicing several times
- No longer subtracts engine hours and age on first load
- Fixed a bug with "bulk buy mod" (pallet could not be bought)
- Fixed multiplayer problems


Changelog Version 1.0.0.2:
- FPS problems fixed
- Dedicated server problems fixed
- Lamp light removed
- Description slightly adjusted
- Fixed bug where the button did not appear
- Fixed bug where the old state of the vehicle was not restored


Changelog Version 1.0.0.3:
- Description adapted
- Fixed a bug where the button has still not appeared.
- The remaining time needed for maintenance is now saved and continued when re-entering the savegame
- Maintenance can now be done at any time
- Maintenance is now only necessary 1 time per season with the Seasons-Mod
- At the pallet it is now possible to configure how long it takes. until the maintenance is completed. (Less time = higher costs for maintenance)
- Maintenance now costs a certain amount
- Default damage from Ls19 adjusts now to the maintenance
--]]

-- Thanks to Ian for the help with the xml!
-- Thanks to Glowin for the help with the last bugs and for the lua with the server stuff!

ExtendedVehicleMaintenance = {};
ExtendedVehicleMaintenance.rootNodePallet = {};
ExtendedVehicleMaintenance.l10nEnv = "FS19_ExtendedVehicleMaintenance";

function ExtendedVehicleMaintenance.prerequisitesPresent(specializations)
	return true;
end;

function ExtendedVehicleMaintenance.registerEventListeners(vehicleType)
	SpecializationUtil.registerEventListener(vehicleType, "onLoad", ExtendedVehicleMaintenance);
	SpecializationUtil.registerEventListener(vehicleType, "onPostLoad", ExtendedVehicleMaintenance);
	SpecializationUtil.registerEventListener(vehicleType, "onReadStream", ExtendedVehicleMaintenance);
	SpecializationUtil.registerEventListener(vehicleType, "onWriteStream", ExtendedVehicleMaintenance);
	SpecializationUtil.registerEventListener(vehicleType, "onReadUpdateStream", ExtendedVehicleMaintenance);
	SpecializationUtil.registerEventListener(vehicleType, "onWriteUpdateStream", ExtendedVehicleMaintenance);
	SpecializationUtil.registerEventListener(vehicleType, "onUpdate", ExtendedVehicleMaintenance);
end;

function ExtendedVehicleMaintenance:onLoad(savegame)
	local spec = self.spec_ExtendedVehicleMaintenance

	if self.spec_motorized ~= nil then

		spec.Variable = 30.0;
		spec.VariableBackupCheck = 0;

		spec.WartezeitStunden = 2;
		
		spec.WartezeitMinuten = 60;

		spec.CurrentMinuteBackup = g_currentMission.hud.environment.currentMinute;

		spec.Minus = 30;

		spec.WartungKnopfGedrückt = false;

		spec.Differenz = 0;

		spec.MotorDieTimer = -1;
		
		spec.BackupOperatingTimeXML = 0;
		
		spec.DontAllowXmlNumberReset = false;
		
		spec.MaintenanceTimes = 1;
		
		spec.DarfNichtAusgehen = false;
		
		spec.RandomNumber = 0;
		
		spec.SecondSound = 2;
		
		spec.NumberMotorDieTimer = 0;
		
		spec.Costs = 0;
		
		spec.Amount = 0;
		
		spec.AmountBeforeMinus = 0;
		
		-- days

		spec.Days = 36;

		spec.SeasonsDays = 36;

		spec.BackupAge = 0;

		spec.BackupAgeXML = 0;

		spec.DifferenzDays = 0;
		
		spec.dirtyFlag = self:getNextDirtyFlag()
	end;
	g_currentMission.missionInfo.automaticMotorStartEnabled = false
	g_currentMission.inGameMenu.pageSettingsGame.checkAutoMotorStart:setVisible(false)    
end;

function ExtendedVehicleMaintenance:onPostLoad(savegame)
	local spec = self.spec_ExtendedVehicleMaintenance

	if savegame ~= nil then
		spec.BackupAgeXML = Utils.getNoNil(getXMLInt(savegame.xmlFile, savegame.key .. ".ExtendedVehicleMaintenance#BackupAgeXML"), spec.BackupAgeXML)
		spec.BackupOperatingTimeXML = Utils.getNoNil(getXMLInt(savegame.xmlFile, savegame.key .. ".ExtendedVehicleMaintenance#backupOperatingTime"), spec.BackupOperatingTimeXML)
		spec.MaintenanceTimes = Utils.getNoNil(getXMLInt(savegame.xmlFile, savegame.key .. ".ExtendedVehicleMaintenance#MaintenanceTimes"), spec.MaintenanceTimes)
		spec.Differenz = Utils.getNoNil(getXMLInt(savegame.xmlFile, savegame.key .. ".ExtendedVehicleMaintenance#Differenz"), spec.Differenz)
		spec.DifferenzDays = Utils.getNoNil(getXMLInt(savegame.xmlFile, savegame.key .. ".ExtendedVehicleMaintenance#DifferenzDays"), spec.DifferenzDays)
		spec.WartezeitStunden = Utils.getNoNil(getXMLInt(savegame.xmlFile, savegame.key .. ".ExtendedVehicleMaintenance#WartezeitStunden"), spec.WartezeitStunden)
		spec.WartezeitMinuten = Utils.getNoNil(getXMLInt(savegame.xmlFile, savegame.key .. ".ExtendedVehicleMaintenance#WartezeitMinuten"), spec.WartezeitMinuten)
		spec.WartungKnopfGedrückt = Utils.getNoNil(getXMLBool(savegame.xmlFile, savegame.key .. ".ExtendedVehicleMaintenance#WartungKnopfGedrückt"), spec.WartungKnopfGedrückt)
		if Utils.getNoNil(getXMLInt(savegame.xmlFile, savegame.key .. ".ExtendedVehicleMaintenance#BackupAgeXML")) ~= nil and Utils.getNoNil(getXMLInt(savegame.xmlFile, savegame.key .. ".ExtendedVehicleMaintenance#backupOperatingTime")) ~= nil then
	        spec.DontAllowXmlNumberReset = true
	    end;
	end

end

function ExtendedVehicleMaintenance:saveToXMLFile(xmlFile, key, usedModNames)
	local spec = self.spec_ExtendedVehicleMaintenance

	if spec.BackupAgeXML ~= nil then
		setXMLInt(xmlFile, key .. "#BackupAgeXML", spec.BackupAgeXML)
	end
	if spec.BackupOperatingTimeXML ~= nil then
		setXMLInt(xmlFile, key .. "#backupOperatingTime", spec.BackupOperatingTimeXML)
	end	
	if spec.MaintenanceTimes ~= nil then
		setXMLInt(xmlFile, key .. "#MaintenanceTimes", spec.MaintenanceTimes)
	end
    if spec.Differenz ~= nil then
		setXMLInt(xmlFile, key .. "#Differenz", spec.Differenz)
	end
	if spec.DifferenzDays ~= nil then
		setXMLInt(xmlFile, key .. "#DifferenzDays", spec.DifferenzDays)
	end
	if spec.WartezeitStunden ~= nil then
		setXMLInt(xmlFile, key .. "#WartezeitStunden", spec.WartezeitStunden)
	end
	if spec.WartezeitMinuten ~= nil then
		setXMLInt(xmlFile, key .. "#WartezeitMinuten", spec.WartezeitMinuten)
	end
	if spec.WartungKnopfGedrückt ~= nil then
		setXMLBool(xmlFile, key .. "#WartungKnopfGedrückt", spec.WartungKnopfGedrückt)
	end
end

function ExtendedVehicleMaintenance:onReadStream(streamId, connection)
	local spec = self.spec_ExtendedVehicleMaintenance
	local motorized = self.spec_motorized ~= nil
	 if motorized then
		spec.BackupAgeXML = streamReadInt32(streamId)
		spec.BackupOperatingTimeXML = streamReadInt32(streamId)
		spec.MaintenanceTimes = streamReadInt32(streamId)
		spec.Differenz = streamReadInt32(streamId)
		spec.DifferenzDays = streamReadInt32(streamId)
		spec.WartezeitStunden = streamReadInt32(streamId)
		spec.WartezeitMinuten = streamReadInt32(streamId)
		spec.DontAllowXmlNumberReset = streamReadBool(streamId)
		spec.WartungKnopfGedrückt = streamReadBool(streamId)
	end
end

function ExtendedVehicleMaintenance:onWriteStream(streamId, connection)
	local spec = self.spec_ExtendedVehicleMaintenance
	local motorized = self.spec_motorized ~= nil
	
	if motorized then
		streamWriteInt32(streamId, spec.BackupAgeXML)
		streamWriteInt32(streamId, spec.BackupOperatingTimeXML)
		streamWriteInt32(streamId, spec.MaintenanceTimes)
		streamWriteInt32(streamId, spec.Differenz)
		streamWriteInt32(streamId, spec.DifferenzDays)
		streamWriteInt32(streamId, spec.WartezeitStunden)
		streamWriteInt32(streamId, spec.WartezeitMinuten)
		streamWriteBool(streamId, spec.DontAllowXmlNumberReset)
		streamWriteBool(streamId, spec.WartungKnopfGedrückt)
	end
end

function ExtendedVehicleMaintenance:onReadUpdateStream(streamId, timestamp, connection)
	if not connection:getIsServer() then
		local spec = self.spec_ExtendedVehicleMaintenance
		
		if streamReadBool(streamId) then
			spec.BackupAgeXML = streamReadInt32(streamId)
			spec.BackupOperatingTimeXML = streamReadInt32(streamId)
			spec.MaintenanceTimes = streamReadInt32(streamId)
			spec.Differenz = streamReadInt32(streamId)
			spec.DifferenzDays = streamReadInt32(streamId)
			spec.WartezeitStunden = streamReadInt32(streamId)
			spec.WartezeitMinuten = streamReadInt32(streamId)
			spec.DontAllowXmlNumberReset = streamReadBool(streamId)
			spec.WartungKnopfGedrückt = streamReadBool(streamId)
		end
	end
end

function ExtendedVehicleMaintenance:onWriteUpdateStream(streamId, connection, dirtyMask)
	if connection:getIsServer() then
		local spec = self.spec_ExtendedVehicleMaintenance
		if spec.dirtyFlag ~= nil then
			if streamWriteBool(streamId, bitAND(dirtyMask, spec.dirtyFlag) ~= 0) then
				streamWriteInt32(streamId, spec.BackupAgeXML)
				streamWriteInt32(streamId, spec.BackupOperatingTimeXML)
				streamWriteInt32(streamId, spec.MaintenanceTimes)
				streamWriteInt32(streamId, spec.Differenz)
				streamWriteInt32(streamId, spec.DifferenzDays)
				streamWriteInt32(streamId, spec.WartezeitStunden)
				streamWriteInt32(streamId, spec.WartezeitMinuten)
				streamWriteBool(streamId, spec.DontAllowXmlNumberReset)
				streamWriteBool(streamId, spec.WartungKnopfGedrückt)
			end
		else 
			streamWriteBool(streamId, false)
		end
	end
end

function ExtendedVehicleMaintenance:onUpdate(dt)
    g_directSellDialog.repairButton.disabled = true
	if self.typeName == "FS19_ExtendedVehicleMaintenance.palletMaintencance" then -- searches for the Pallet
		ExtendedVehicleMaintenance.rootNodePallet[self.id] = self.rootNode;
		if self.configurations.design == 1 then
		    OriginalTime = 1
		    Costs = 25000
		elseif self.configurations.design  == 2 then
		    OriginalTime = 4
		    Costs = 20000
		elseif self.configurations.design == 3 then
		    OriginalTime = 8
		    Costs = 16000
		elseif self.configurations.design == 4 then
		    OriginalTime = 24
		    Costs = 12000
		elseif self.configurations.design == 5 then
		    OriginalTime = 48
		    Costs = 8000
		end
		
	end;
	
	if self.spec_motorized ~= nil then
		local changeFlag = false
	    if self.isClient and self:getIsActiveForInput() and self.spec_ExtendedVehicleMaintenance.DontAllowXmlNumberReset ~= true then
	        self.spec_ExtendedVehicleMaintenance.BackupAgeXML = self.age
	        self.spec_ExtendedVehicleMaintenance.BackupOperatingTimeXML = self:getFormattedOperatingTime()
		    self.spec_ExtendedVehicleMaintenance.DontAllowXmlNumberReset = true
			changeFlag = true
	    end;
		if self.isClient and self:getIsActiveForInput() and self.spec_ExtendedVehicleMaintenance.Variable < 0.05 or self.spec_ExtendedVehicleMaintenance.Days <= 0 then -- checks if a value is under 0
			Wartung = true
		end;
		if self.isClient and self:getIsActiveForInput() and self.spec_ExtendedVehicleMaintenance.Variable >= 0.05 and self.spec_ExtendedVehicleMaintenance.Days > 0 then
			Wartung = false
		end;

		if self.isClient and self:getIsActiveForInput() and Wartung == false and self.spec_ExtendedVehicleMaintenance.WartungKnopfGedrückt == false then
			g_currentMission:addExtraPrintText(g_i18n:getText("information_Motor", ExtendedVehicleMaintenance.l10nEnv):format(self.spec_ExtendedVehicleMaintenance.Variable, self.spec_ExtendedVehicleMaintenance.Days)) -- 110n Text
		end;
		--hours
		if self.isClient and self:getIsActiveForInput() then
			self.spec_ExtendedVehicleMaintenance.Variable = (30 * self.spec_ExtendedVehicleMaintenance.MaintenanceTimes) - self:getFormattedOperatingTime() + self.spec_ExtendedVehicleMaintenance.Differenz + self.spec_ExtendedVehicleMaintenance.BackupOperatingTimeXML-- Operating time number
			--days
			self.spec_ExtendedVehicleMaintenance.Days = (self.spec_ExtendedVehicleMaintenance.SeasonsDays * self.spec_ExtendedVehicleMaintenance.MaintenanceTimes) - self.age + self.spec_ExtendedVehicleMaintenance.DifferenzDays + self.spec_ExtendedVehicleMaintenance.BackupAgeXML -- day number
			
			--wear
		     self.spec_wearable.totalAmount = self.spec_ExtendedVehicleMaintenance.Amount
			--amount is
			
			if self.spec_ExtendedVehicleMaintenance.Days < self.spec_ExtendedVehicleMaintenance.Variable then
			    self.spec_ExtendedVehicleMaintenance.AmountBeforeMinus = self.spec_ExtendedVehicleMaintenance.Days * 100 / self.spec_ExtendedVehicleMaintenance.SeasonsDays
				self.spec_ExtendedVehicleMaintenance.Amount = (100 - self.spec_ExtendedVehicleMaintenance.AmountBeforeMinus) / 100
			else 
			    self.spec_ExtendedVehicleMaintenance.AmountBeforeMinus = self.spec_ExtendedVehicleMaintenance.Variable * 100 / 30
				self.spec_ExtendedVehicleMaintenance.Amount = (100 - self.spec_ExtendedVehicleMaintenance.AmountBeforeMinus) / 100
			end		
		end;
       
		if g_currentMission.environment.weather.environment ~= nil then
			self.spec_ExtendedVehicleMaintenance.SeasonsDays = g_currentMission.environment.weather.environment.daysPerSeason * 4
		else
			self.spec_ExtendedVehicleMaintenance.SeasonsDays = 36
		end;
		
		-- Action if number is 0
		if self.isClient and self:getIsActiveForInput() and Wartung == true and self.spec_ExtendedVehicleMaintenance.WartungKnopfGedrückt ~= true then
			g_currentMission:addExtraPrintText(g_i18n:getText("warning_wartung", ExtendedVehicleMaintenance.l10nEnv))

			if self.spec_ExtendedVehicleMaintenance.RandomNumber == 0 then
				self.spec_ExtendedVehicleMaintenance.RandomNumber = math.random(1, 2)
			end;

			if self.spec_ExtendedVehicleMaintenance.DarfNichtAusgehen ~= true then

				--or self:getMotorLoadPercentage() >= 0.9999
				if self:getIsMotorStarted() then
				    if self.spec_motorized.samples.motorStart ~= nil then
					    self.spec_ExtendedVehicleMaintenance.MotorDieTimer = self.spec_ExtendedVehicleMaintenance.MotorDieTimer - dt
					    self.spec_ExtendedVehicleMaintenance.NumberMotorDieTimer = math.min(-self.spec_ExtendedVehicleMaintenance.MotorDieTimer / 2000, 0.9)
					    -- print(self.spec_ExtendedVehicleMaintenance.NumberMotorDieTimer)
					    local MaxNumber = self.spec_motorized.samples.motorStart.duration * 98 / 1000000
					    if self.spec_ExtendedVehicleMaintenance.NumberMotorDieTimer >= 0.290789 then
					    	g_soundManager:stopSample(self.spec_motorized.samples.motorStart)
					    	self.spec_ExtendedVehicleMaintenance.MotorDieTimer = -1
						    g_soundManager:playSample(self.spec_motorized.samples.motorStart)
						    self.spec_ExtendedVehicleMaintenance.SecondSound = self.spec_ExtendedVehicleMaintenance.SecondSound - 1
					    end;

					    if self.spec_ExtendedVehicleMaintenance.RandomNumber == 1 and self.spec_ExtendedVehicleMaintenance.NumberMotorDieTimer >= 0.290789 and self.spec_ExtendedVehicleMaintenance.SecondSound == 0 then
						    self:stopMotor()
						    g_soundManager:stopSample(self.spec_motorized.samples.motorStop)
						    self.spec_ExtendedVehicleMaintenance.SecondSound = 2
						    self.spec_ExtendedVehicleMaintenance.RandomNumber = 0
					    elseif self.spec_ExtendedVehicleMaintenance.RandomNumber == 2 and self.spec_ExtendedVehicleMaintenance.NumberMotorDieTimer >= 0.290789 and self.spec_ExtendedVehicleMaintenance.SecondSound == 1 then
						    self.spec_ExtendedVehicleMaintenance.DarfNichtAusgehen = true
						    self.spec_ExtendedVehicleMaintenance.RandomNumber = 0
						    self.spec_ExtendedVehicleMaintenance.SecondSound = 2
					    end;
					end;
				end;
			end;
			if self:getIsMotorStarted() and OnlyWhenIsGettingActive == false then
				self.spec_ExtendedVehicleMaintenance.DarfNichtAusgehen = true
				OnlyWhenIsGettingActive = true
			else
				OnlyWhenIsGettingActive = true
			end;

			if self:getMotorLoadPercentage() > 0.999999 then
				self.spec_motorized.smoothedLoadPercentage = 0
				self:stopMotor()
			end;
			if self.spec_motorized.isMotorStarted == false then
				self.spec_ExtendedVehicleMaintenance.DarfNichtAusgehen = false
			end;
		end;
		if Wartung == false then
			OnlyWhenIsGettingActive = false
		end;

		-- looks for the position of the pallet and adds the action event
		if self.isClient and self:getIsActiveForInput() then
		
			local paletteGefunden = false
			
			for id,rootNode in pairs(ExtendedVehicleMaintenance.rootNodePallet) do 
			    --print(id)
				
			    if g_currentMission.nodeToObject[rootNode] ~= nil then
			        local px, py, pz = getWorldTranslation(rootNode) -- Palette
			        local vx, vy, vz = getWorldTranslation(self.rootNode) -- Fahrzeug
				
				
				    if MathUtil.vector3Length(px-vx, py-vy, pz-vz) < 6 then		
				        paletteGefunden = true
			        end
				end
			end
			
			if paletteGefunden then
			    local spec = self.spec_ExtendedVehicleMaintenance;
				spec.actionEvents = {};
				self:clearActionEventsTable(spec.actionEvents);


				local _, actionEventIdWartung = self:addActionEvent(spec.actionEvents, InputAction.VEHICLE_MAINTENANCE, self, ExtendedVehicleMaintenance.VEHICLE_MAINTENANCE, false, true, false, true, nil);

				g_inputBinding:setActionEventTextPriority(actionEventIdWartung, GS_PRIO_NORMAL)
			else
			    local spec = self.spec_ExtendedVehicleMaintenance;
				self:removeActionEvent(spec.actionEvents, InputAction.VEHICLE_MAINTENANCE)
			end
		end;
		
		-- action if action event input is pressed
		if self.spec_ExtendedVehicleMaintenance.WartungKnopfGedrückt == true then
			local spec = self.spec_ExtendedVehicleMaintenance
			self:stopMotor()
			changeFlag = true
			-- subtracts the time from the 1 hour 
			if self.spec_ExtendedVehicleMaintenance.CurrentMinuteBackup ~= g_currentMission.hud.environment.currentMinute then
				self.spec_ExtendedVehicleMaintenance.CurrentMinuteBackup = g_currentMission.hud.environment.currentMinute
				self.spec_ExtendedVehicleMaintenance.WartezeitMinuten = self.spec_ExtendedVehicleMaintenance.WartezeitMinuten - 1
			end;
			if self.spec_ExtendedVehicleMaintenance.WartezeitMinuten < 0 then
			    self.spec_ExtendedVehicleMaintenance.WartezeitMinuten = 59
			    self.spec_ExtendedVehicleMaintenance.WartezeitStunden = self.spec_ExtendedVehicleMaintenance.WartezeitStunden - 1
			end
			if self.isClient and self:getIsActiveForInput() then
				g_currentMission:addExtraPrintText(g_i18n:getText("information_wartung", ExtendedVehicleMaintenance.l10nEnv):format(self.spec_ExtendedVehicleMaintenance.WartezeitStunden, self.spec_ExtendedVehicleMaintenance.WartezeitMinuten))
			end;
			self:removeActionEvent(spec.actionEvents, InputAction.VEHICLE_MAINTENANCE)
		end;
		-- action if 1 hour is over 
		if self.spec_ExtendedVehicleMaintenance.WartezeitStunden <= -1 and self.spec_ExtendedVehicleMaintenance.WartungKnopfGedrückt == true then
			self.spec_ExtendedVehicleMaintenance.Minus = self.spec_ExtendedVehicleMaintenance.Minus + 30
			self.spec_ExtendedVehicleMaintenance.BackupAge = self.age
			self.spec_ExtendedVehicleMaintenance.WartungKnopfGedrückt = false
			self.spec_ExtendedVehicleMaintenance.BackupOperatingTimeXML = 0
			self.spec_ExtendedVehicleMaintenance.BackupAgeXML = 0
			self.spec_ExtendedVehicleMaintenance.MaintenanceTimes = self.spec_ExtendedVehicleMaintenance.MaintenanceTimes + 1
			self.spec_ExtendedVehicleMaintenance.Differenz = self:getFormattedOperatingTime() - (30 * (self.spec_ExtendedVehicleMaintenance.MaintenanceTimes - 1))
			self.spec_ExtendedVehicleMaintenance.DifferenzDays = self.age - (self.spec_ExtendedVehicleMaintenance.SeasonsDays * (self.spec_ExtendedVehicleMaintenance.MaintenanceTimes - 1))
			changeFlag = true
		end;
		if changeFlag then
			self:raiseDirtyFlags(self.spec_ExtendedVehicleMaintenance.dirtyFlag)
		end
	end;
end;

function ExtendedVehicleMaintenance:VEHICLE_MAINTENANCE()
	self.spec_ExtendedVehicleMaintenance.WartezeitMinuten = 0
	self.spec_ExtendedVehicleMaintenance.WartezeitStunden = OriginalTime
	self.spec_ExtendedVehicleMaintenance.CurrentMinuteBackup = g_currentMission.hud.environment.currentMinute
	self.spec_ExtendedVehicleMaintenance.WartungKnopfGedrückt = true
	g_currentMission:consoleCommandCheatMoney(-Costs)
end;